package org.colcon.burocrat.api.utils;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import com.google.gwt.regexp.shared.MatchResult;
import com.google.gwt.regexp.shared.RegExp;

public class StringUtil 
{
	public static final int
	
		MIN_ID_LENGTH = 3,
		MAX_ID_LENGTH = 128,
		MIN_PASSWORD_LENGTH = 3,
		MAX_PASSWORD_LENGTH = 32,
		MAX_TEXT_LENGTH = 2000;
	
	public static final String
	
		ID_REGEX = "^[a-zA-Z0-9_\\.]*$",
		SUBPATH_REGEX = ".*";
	 
	public static boolean matches(String str, String regex)
	{
		RegExp regExp = RegExp.compile(regex);
		MatchResult matcher = regExp.exec(str);
		return matcher != null;
	}
	
	public static boolean hasSpecialCharacters(String str)
	{
		return !matches(str, ID_REGEX);
	}
	
	public static boolean isValidPath(String str)
	{
		if(str == null || str.length() < MIN_ID_LENGTH)
			return false;
		
		for(String id : str.split("/"))
			if(!isValidId(id))
				return false;
	
		return true;
	}
	
	public static boolean isValidId(String str)
	{
		return 
				str != null && 
				str.length() >= MIN_ID_LENGTH &&
				str.length() <= MAX_ID_LENGTH &&
				!hasSpecialCharacters(str);
				
	}
	
	public static String[] getHashtagPaths(String str)
	{
		List<String> list = new LinkedList<String>();
		
		for(String word : str.split(" "))
		{
			if(word.startsWith("#"))
				list.add(word.substring(1));
		}
		
		return list.toArray(new String[0]);
	}
	
	public static String randomString(int length)
	{
		StringGenerator generator = new StringUtil.StringGenerator(length);
		return generator.nextString();
	}
	
	public static String encodeURLSafe(String url)
	{
		url.replace("!", "%21");
		url.replace("#", "%23");
		url.replace("$", "%24");
		url.replace("&", "%26");
		url.replace("'", "%27");
		url.replace("(", "%28");
		url.replace(")", "%29");
		url.replace("*", "%2A");
		url.replace("+", "%2B");
		url.replace(",", "%2C");
		url.replace("/", "%2F");
		url.replace(":", "%3A");
		url.replace(";", "%3B");
		url.replace("=", "%3D");
		url.replace("?", "%3F");
		url.replace("@", "%40");
		url.replace("[", "%5B");
		url.replace("]", "%5D");
		return url;
	}
	
	private static class StringGenerator
	{
		private static final char[] symbols = new char[36];
	
		static
		{
			for (int idx = 0; idx < 10; ++idx)
				symbols[idx] = (char) ('0' + idx);
			for (int idx = 10; idx < 36; ++idx)
				symbols[idx] = (char) ('a' + idx - 10);
		}
	
		private final Random random = new Random();
	
		private final char[] buf;
	
		public StringGenerator(int length)
		{
			if (length < 1)
				throw new IllegalArgumentException("length < 1: " + length);
			buf = new char[length];
		}
	
		public String nextString()
		{
			for (int idx = 0; idx < buf.length; ++idx)
					buf[idx] = symbols[random.nextInt(symbols.length)];
			return new String(buf);
		}
	}
}
