/**
 * 
 */
package org.colcon.burocrat.client.form;

import java.util.ArrayList;

import org.colcon.burocrat.api.client.IClient;
import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.client.BurocratField;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * A BurocratField used for inputing a dynamic list of values
 * 
 * @author Jesse Freitas
 * 
 */
public class ListField extends BurocratField
{

	private ArrayList<TextBox> choices = new ArrayList<TextBox>();
	private Button addChoice = new Button("Add");
	private Button removeChoice = new Button("Remove");
	private VerticalPanel choicesPane = new VerticalPanel();
	private VerticalPanel rootPane = new VerticalPanel();
	private HorizontalPanel modifyPane = new HorizontalPanel();
	private int num_choices = 0;

	/**
	 * @param client
	 * @param arg
	 */
	public ListField(IClient client, IArg arg)
	{
		super(client, arg);
	}

	public void addItem()
	{
		TextBox t = new TextBox();
		choices.add(t);
		choicesPane.add(t);
		num_choices++;
	}

	public void removeItem()
	{
		choicesPane.remove(choices.remove(choices.size() - 1));
		num_choices--;
	}

	public String[] getValues()
	{
		String[] toRtn = new String[num_choices];
		for (int i = 0; i < num_choices; i++)
		{
			toRtn[i] = choices.get(i).getValue();
		}
		return toRtn;
	}

	/**
	 * @see org.colcon.burocrat.client.BurocratField#getValue()
	 */
	@Override
	public String getValue()
	{
		String[] arr = getValues();
		String toRtn = "";

		for (int i = 0; i < arr.length - 1; i++)
		{
			if (arr[i] != "")
				toRtn += arr[i] + ",";
		}
		if (arr.length > 0)
			toRtn += arr[arr.length - 1];

		return toRtn;
	}

	/**
	 * @see org.colcon.burocrat.client.BurocratField#setValue(java.lang.String)
	 */
	@Override
	public void setValue(String value)
	{

	}

	/**
	 * @see org.colcon.burocrat.client.ui.BurocratPanel#init(org.colcon.burocrat.api.model.IResponse)
	 */
	@Override
	public void init(IResponse data)
	{
		addChoice.addClickHandler(new ClickHandler()
		{
			@Override
			public void onClick(ClickEvent event)
			{
				TextBox t = new TextBox();
				choices.add(t);
				choicesPane.add(t);
				num_choices++;
			}
		});
		removeChoice.addClickHandler(new ClickHandler()
		{
			@Override
			public void onClick(ClickEvent event)
			{
				if (choices.size() != 0)
				{
					choicesPane.remove(choices.remove(choices.size() - 1));
					num_choices--;
				}
			}

		});
		modifyPane.add(addChoice);
		modifyPane.add(removeChoice);
		rootPane.add(choicesPane);
		rootPane.add(modifyPane);
		add(rootPane);
	}

}
