package org.colcon.burocrat.client.item;

import org.colcon.burocrat.api.client.IClient;
import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IItem;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.client.CSS;
import org.colcon.burocrat.client.ui.BurocratPage;
import org.colcon.burocrat.model.Model;

import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * 
 * A basic view of the raw data stored in an item,
 * consisting of a table of key-value pairs
 * 
 * @author Dan Vegeto
 *
 */
public class ItemPage extends BurocratPage
{
	private IItem item;
	
	public ItemPage(IClient client, IItem item) 
	{
		super(client);
		
		this.item = item;
	}
	
	public IItem getItem()
	{
		return item;
	}
	
	public void init(IResponse data)
	{
		super.init(data);
		
		VerticalPanel panel = new VerticalPanel();
		ScrollPanel scroll = new ScrollPanel(panel);
		
		panel.setWidth("100%");
		scroll.setWidth("100%");
		
		panel.add(buildHeader());
		panel.add(buildPage());
		
		add(scroll);
	}
	
	public Widget buildHeader()
	{
		HorizontalPanel header = new HorizontalPanel();
		
		String[] pathArray = Model.instance.parser().parsePath(item.getPath());
		
		// build links to each item in ancestry
		for(int i = 0; i < pathArray.length; i++)
		{
			String[] subArray = new String[i + 1];
			
			for(int j = 0; j < i + 1; j++)
				subArray[j] = pathArray[j];
		
			String link = Model.instance.parser().composePath(subArray);
			String id = subArray[subArray.length - 1];
			
			Hyperlink hyperlink = new Hyperlink(id, link);
			
			hyperlink.setStyleName(CSS.ITEM_PATH.getStyle());
			
			if(i > 0)
			{
				Label delim = new Label("/");
				delim.setStyleName(CSS.ITEM_PATH.getStyle());
				header.add(delim);
			}
				
			header.add(hyperlink);
		}
		
		return header;
	}
	
	public Widget buildPage()
	{
		VerticalPanel panel = new VerticalPanel();
		
		panel.setStyleName(CSS.ITEM_PAGE.getStyle());
		
		Grid grid = new Grid(0, 2);
		grid.setStyleName(CSS.ITEM_GRID.getStyle());
		
		for(IArg arg : item.getArgs().keySet())
		{
			HTML key = new HTML(arg.toString());
			HTML value = new HTML(item.getArg(arg));
			
			key.setStyleName(CSS.ITEM_KEY.getStyle());
			value.setStyleName(CSS.ITEM_VALUE.getStyle());
			
			grid.resizeRows(grid.getRowCount() + 1);
			
			grid.setWidget(grid.getRowCount() - 1, 0, key);
			grid.setWidget(grid.getRowCount() - 1, 1, value);
		}
		
		panel.add(grid);
		
		return panel;
	}
}
