package org.colcon.burocrat.client.pages;

import java.util.Map;

import org.colcon.burocrat.api.client.IClient;
import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.ICommand;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.client.BurocratForm;
import org.colcon.burocrat.client.CSS;
import org.colcon.burocrat.client.ui.BurocratPage;
import org.colcon.burocrat.model.Arg;
import org.colcon.burocrat.model.Cmd;
import org.colcon.burocrat.model.Command;
import org.colcon.burocrat.model.Model;

import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * 
 * This is a page that allows users to manually define a server request with a BurocratForm. 
 * The current state of the form data is stored within the history token, allowing form progress
 * to be saved in the as a hyperlink. 
 * 
 * @author Dan Vegeto
 *
 */
public class FormPage extends BurocratPage implements ChangeHandler, ClickHandler
{
	private BurocratForm form;
	private Button button;
	private HorizontalPanel status;
	
	public FormPage(IClient client)
	{
		super(client);
	}

	public void init(IResponse data)
	{
		super.init(data);
		
		VerticalPanel panel = new VerticalPanel();
		ScrollPanel scroll = new ScrollPanel(panel);
		
		panel.setWidth("100%");
		panel.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
		
		form = new BurocratForm(getClient(), Arg.COMMAND);
		form.addChangeHandler(this);
		
		button = new Button("submit");
		button.setStyleName(CSS.TEXT.getStyle());
		button.addClickHandler(this);
		
		status = new HorizontalPanel();
		
		panel.add(form);
		panel.add(status);
		panel.add(button);
		
		add(scroll);
		
		form.setValue(getClient().getArgs());
	}
	
	public void onChange(ChangeEvent event) 
	{
		Map<IArg, String> values = form.getValueMap();
		
		updateURLArgs(values);
		updateFormStatus(values);
	}
	
	private void updateURLArgs(Map<IArg, String> values)
	{
		Map<IArg, String> clientArgs = getClient().getArgs();
		
		boolean different = false;
		
		for(IArg key : values.keySet())
		{
			if(!values.get(key).equalsIgnoreCase(clientArgs.get(key)))
			{
				different = true;
				break;
			}
		}
		
		if(different)
			getClient().setArgs(values);
	}

	private void updateFormStatus(Map<IArg, String> values)
	{
		String value = form.getValue();
		
		ICommand command = new Command(
				Arg.CMD.put(Cmd.VALIDATE),
				Arg.COMMAND.put(value));
		
		status.clear();
		
		if(value == null || value.isEmpty())
			return;
		
		status.add(getClient().widgets().loading());
		
		getClient().sendRequest(command, new AsyncCallback<IResponse>()
		{
			public void onFailure(Throwable caught) 
			{
				status.clear();
				status.add(new HTML("uh oh, something went wrong (try refreshing)"));
			}

			public void onSuccess(IResponse result) 
			{
				status.clear();
				
				if(result.isSuccess())
					status.add(form.getValidStatus());
				else
				{
					status.add(form.getInvalidStatus());
					status.add(new HTML("<pre>" + result + "</pre>"));
				}
			}
		});
	}
	
	public void onClick(ClickEvent event) 
	{
		String command = form.getValue();
		ICommand commandObj = Model.instance.parser().parseCommand(command);
		
		button.setEnabled(false);
		
		final PopupPanel popup = new PopupPanel(true);
		
		popup.setStyleName(CSS.FORM_POPUP.getStyle());
		
		popup.add(getClient().widgets().loading());
		popup.center();
		popup.show();
		
		getClient().sendRequest(commandObj, new AsyncCallback<IResponse>()
		{
			public void onFailure(Throwable caught) 
			{
				popup.clear();
				popup.add(new HTML("uh oh, something went wrong (try refreshing)"));
				popup.center();
				button.setEnabled(true);
			}

			public void onSuccess(IResponse result) 
			{
				popup.clear();
				
				popup.add(new HTML("<pre>" + result.getMessage() + "</pre>"));
				popup.center();
				button.setEnabled(true);
			}
		});
	}
}
