package org.colcon.burocrat.client.pages;

import java.util.LinkedList;
import java.util.List;

import org.colcon.burocrat.api.client.IClient;
import org.colcon.burocrat.api.model.ICommand;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.client.CSS;
import org.colcon.burocrat.client.ui.BurocratPage;
import org.colcon.burocrat.model.Model;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyDownEvent;
import com.google.gwt.event.dom.client.KeyDownHandler;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * This is a debug terminal for the command line interface, 
 * which allows developers to test server requests that are
 * equivalent to those made by the site's GUI
 * 
 * @author Dan Vegeto
 *
 */
public class TerminalPage extends BurocratPage implements ClickHandler, KeyDownHandler
{
	private static final int
	
		FOOTER_HEIGHT = 40;
	
	private List<String> history;
	private int pointer;
	
	private ScrollPanel scroll;
	private VerticalPanel output;
	
	private HorizontalPanel bar;
	private TextBox input;
	private Button button;
	
	public TerminalPage(IClient client) 
	{
		super(client);	
	}
	
	public void init(IResponse data)
	{
		super.init(data);
		
		setStyleName(CSS.TERMINAL.getStyle());
		
		history = new LinkedList<String>();
		pointer = -1;
		
		output = new VerticalPanel();
		scroll = new ScrollPanel(output);
		
		bar = new HorizontalPanel();
		input = new TextBox();
		button = new Button("submit");
		
		input.setStyleName(CSS.TERMINAL_INPUT.getStyle());
		button.setStyleName(CSS.TERMINAL_BUTTON.getStyle());
		
		scroll.setHeight("100%");
		
		bar.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
		bar.setWidth("100%");
		input.setWidth("95%");
		button.setWidth("80%");
		
		bar.add(input);
		bar.add(button);
		
		addSouth(bar, FOOTER_HEIGHT);
		add(scroll);
		
		print("enter 'help' for a list of commands");
		
		button.addClickHandler(this);
		
		sinkEvents(Event.ONKEYDOWN);
		addHandler(this, KeyDownEvent.getType());
		
		input.getElement().focus();
	}
	
	public void onClick(ClickEvent event) 
	{
		buttonClick();
	}
	
	public void onKeyDown(KeyDownEvent event) 
	{
		if(event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
			buttonClick();
		else if(event.getNativeKeyCode() == KeyCodes.KEY_UP)
			up();
		else if(event.getNativeKeyCode() == KeyCodes.KEY_DOWN)
			down();
	}
	
	private void addToHistory(String command)
	{
		pointer = -1;
		if(history.isEmpty() || !history.get(0).equals(command))
			history.add(0, command);
	}
	
	private void up()
	{
		if(history.isEmpty())
			return;
		
		if(pointer + 1 < history.size())
		{
			pointer++;
			input.setText(history.get(pointer));
		}
	}
	
	private void down()
	{
		if(history.isEmpty())
			return;
		
		if(pointer >= 0)
		{
			pointer--;
				
			if(pointer >= 0)
				input.setText(history.get(pointer));
			else
				input.setText("");
		}
	}
	
	public void buttonClick()
	{
		if(!button.isEnabled())
			return;
		
		String command = input.getText();
		input.setText("");
		
		if(command == null || command.isEmpty())
		{
			print(">");
			reset();
			return;
		}
		
		print("> " + command);
		
		ICommand commandObj = Model.instance.parser().parseCommand(command);
		
		if(commandObj == null)
		{
			print("> invalid command '" + command + "'");
			reset();
			return;
		}

		button.setEnabled(false);
		
		addToHistory(command);
		
		getClient().sendRequest(commandObj, new AsyncCallback<IResponse>()
		{
			public void onFailure(Throwable caught)
			{
				caught.printStackTrace();
				
				print("uh oh, something went wrong (try refreshing)");
				
				reset();
			}
			
			public void onSuccess(IResponse result)
			{
				if(result != null)
					print(result.toString());
				else
					print("uh oh, something went wrong (try refreshing)");
				
				reset();
			}
		});
	}
	
	private void print(String str)
	{
		output.add(new HTML("<pre>" + str + "</pre>"));
	}
	
	private void reset()
	{
		scroll.scrollToBottom();
		
		button.setEnabled(true);
		
		input.setText("");
		input.getElement().focus();
	}
}
