package org.colcon.burocrat.client.utils;

import java.util.Date;
import com.google.gwt.i18n.client.DateTimeFormat;
/**
 * This class is for date and time operations
 * on the client side.
 * 
 * @author Jesse Freitas
 *
 */
public class TimeUtil {
	/**
	 * Constants that represent units of time in terms of milliseconds
	 */
	public static final long 
		year = 31449600000l,
		month = 2592000000l,
		week = 604800000l,
		day = 86400000l,
		hour = 3600000l,
		minute = 60000l;
	/**
	 * The specified string format for representing Date
	 */
	public static final DateTimeFormat noTimeFormat = DateTimeFormat.getFormat("yyyy.MM.dd");
	public static final DateTimeFormat format = DateTimeFormat.getFormat("yyyy.MM.dd HH:mm:ss");
		/**Conversion Utility Function for parsing a Date from java.util.Date to a string
		 *using a specific date format. 
		 *@Return Returns null if the date is null.
		 */
		public static String dateToString(Date date)
		{
			if(date == null)
				return null;		
			String str = format.format(date);
			return str;
		}
		/**Conversion Utility Function for parsing a string to a java.util.Date
		 *using a specific date format. 
		 *@Return Returns null if the string is null or 
		 *the string is not formatted properly
		 */
		public static Date stringToDate(String str)
		{
			if(str == null || str == "")
				return new Date();
			return format.parse(str);
		}
	/**A function for receiving the relative time difference between 
	 *two dates in the format specified in DATE_FORMAT. 
	 *@Return Returns a string
	 *(EXAMPLE: "# year(s) ago" or "due in # year(s))*/
	public static String getDifference(Date past, Date current){
		
		long i = current.getTime() - past.getTime();
		if(i > 0){
			if(i > year)
			{
				if(i / year == 1)
					return "1 year ago";
				return i / year + " years ago";
			}
			if(i > month)
			{
				if(i / month == 1)
					return "1 month ago";
				return i / month + " months ago";
			}
			if(i > week)
			{
				if(i / week == 1)
					return "1 week ago";
				return i / week + " weeks ago";
			}
			if(i > day)
			{
				if(i / day == 1)
					return "1 day ago";
				return i / day + " days ago";
			}
			if(i > hour)
			{
				if(i / hour == 1)
					return "1 hour ago";
				return i / hour + " hours ago";
			}
			if(i > minute)
			{
				if(i / minute == 1)
					return "1 minute ago";
				return i / minute + " minutes ago";
			}
			return "a few seconds ago";
		}
		if(i < 0){
			if(i < year)
			{
				if(-1 * i / year == 1)
					return "in 1 year";
				return "in " + -1 * i / year + " years";
			}
			if(i < month)
			{
				if(-1 * i / month == 1)
					return "in 1 month";
				return "in " + -1 * i / month + " months";
			}
			if(i < week)
			{
				if(-1 * i / week == 1)
					return "in 1 week";
				return "in " + -1 * i / week + " weeks";
			}
			if(i < day)
			{
				if(-1 * i / day == 1)
					return "in 1 day";
				return "in " + -1 * i / day + " days";
			}
			if(i < hour)
			{
				if(-1 * i / hour == 1)
					return "in 1 hour";
				return "in " + -1 * i / hour + " hours";
			}
			if(i < minute)
			{
				if(-1 * i / minute == 1)
					return "in 1 minute";
				return "in " + -1 * i / minute + " minutes";
			}
			return "in a few seconds";
		}
		return "Time Unavailable";
	}
	/**Using a duration in a long in terms of milliseconds and returns the end date in
	 * relation to the start date. The Start string is in terms of the format DATE_FORMAT 
	 * */
	public static Date getEnd(long duration, String start){
		Date d = stringToDate(start);
		long i = 0;
		if(d != null){
			return new Date(d.getTime() + i);
		}
		return null;
	}
	/**Using a duration in a long in terms of milliseconds and returns the start date in
	 * relation to the end date. The End string is in terms of the format DATE_FORMAT 
	 * */
	public static Date getStart(long duration, String end){
		Date d = stringToDate(end);
		long i = 0;
		if(d != null){
			return new Date(d.getTime() - i);
		}
		return null;
	}
	public static boolean isValidPath(String str){
		if(TimeUtil.stringToDate(str) != null)
			return true;
		return false;
	}
}
