package org.colcon.burocrat.model;

/**
 * These values are different output codes for 
 * the command line interface. For example,
 * if the server response starts with "FAILED",
 * that means the command failed to execute.
 * 
 * @author Dan Vegeto
 *
 */
public enum CmdOutput 
{
	AUTHENTICATION_ERROR,
	MISSING_POWER,
	SUCCEEDED,
	FAILED,
	DID_NOTHING,
	EMPTY;
	
	public static final String
	
		MESSAGE_SEPARATOR = ": ";
	
	public String getMessage(String str)
	{
		return str.substring(name().length() + MESSAGE_SEPARATOR.length());
	}
	
	public String withMessage(String message)
	{
		return name() + MESSAGE_SEPARATOR + message;
	}

	public static String succeededOrFailed(boolean result, String message)
	{
		if(result)
			return SUCCEEDED.withMessage(message);
		else
			return FAILED.withMessage(message);
	}
	
	public static boolean isSuccess(String output)
	{
		return output.startsWith(SUCCEEDED.name());
	}
}
