package org.colcon.burocrat.model;

import java.util.HashMap;
import java.util.Map;

import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.ICmd;
import org.colcon.burocrat.api.model.IItem;
import org.colcon.burocrat.api.model.IItemType;
import org.colcon.burocrat.api.model.IModel;
import org.colcon.burocrat.api.model.IParser;
import org.colcon.burocrat.api.utils.EnumUtil;

/**
 * 
 * The main model class
 * 
 * @author Dan Vegeto
 *
 */
public class Model implements IModel
{
	public static final String version = "0.1.0";
	
	public static IModel instance =
			
		new Model(Arg.values(), Cmd.values(), ItemType.values(), new Parser(), new IItem[] 
		{
			// server metadata
			new Item(
					Arg.TYPE.put(ItemType.METADATA), 
					Arg.VERSION.put(version)),
			
			// top-level containers
			new Item(
					Arg.TYPE.put(ItemType.CONTAINER),
					Arg.ID.put(ItemType.LINK.plural())),
				
			new Item(
					Arg.TYPE.put(ItemType.CONTAINER),
					Arg.ID.put(ItemType.NEWS.plural())),
					
			new Item(
					Arg.TYPE.put(ItemType.CONTAINER),
					Arg.ID.put(ItemType.USER.plural())),
					
			// predefined users
			new Item(
					Arg.TYPE.put(ItemType.USER),
					Arg.ID.put(Constants.ADMIN),
					Arg.PASSWORD.put(Constants.ADMIN_PASSWORD),
					Arg.EMAIL.put(Constants.ADMIN_EMAIL),
					Arg.EMAIL_CONFIRMED.put(Boolean.valueOf(true))),
				
			new Item(
					Arg.TYPE.put(ItemType.USER),
					Arg.ID.put(Constants.GUEST),
					Arg.PASSWORD.put(Constants.GUEST_PASSWORD),
					Arg.EMAIL.put(Constants.GUEST_EMAIL),
					Arg.EMAIL_CONFIRMED.put(Boolean.valueOf(true)))
			
		});
	
	private Map<String, IArg> args;
	private Map<String, ICmd> cmds;
	private Map<String, IItemType> itemTypes;
	
	private IParser parser;
	
	private IItem[] initItems;
	 
	public Model(IArg[] argsArray, ICmd[] cmdsArray, IItemType[] itemTypesArray, IParser parser, IItem[] initItems)
	{
		args = new HashMap<String, IArg>();
		cmds = new HashMap<String, ICmd>();
		itemTypes = new HashMap<String, IItemType>();
		
		EnumUtil.putAll(args, argsArray);
		EnumUtil.putAll(cmds, cmdsArray);
		EnumUtil.putAll(itemTypes, itemTypesArray);
		
		this.parser = parser;
		this.initItems = initItems;
	}
	
	public IArg[] getArgs()
	{
		return args.values().toArray(new IArg[0]);
	}
	
	public ICmd[] getCmds()
	{
		return cmds.values().toArray(new ICmd[0]);
	}
	
	public IItemType[] getItemTypes()
	{
		return itemTypes.values().toArray(new IItemType[0]);
	}
	
	public IArg getArg(String name)
	{
		return args.get(name.toUpperCase());
	}
	
	public ICmd getCmd(String name)
	{
		return cmds.get(name.toUpperCase());
	}
	
	public IItemType getItemType(String name)
	{
		return itemTypes.get(name.toUpperCase());
	}
	
	public IParser parser()
	{
		return parser;
	}
	
	public IItem[] getInitItems()
	{
		return initItems;
	}
}
