package org.colcon.burocrat.app.model;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IFlag;
import org.colcon.burocrat.api.model.IArgList;
import org.colcon.burocrat.api.model.IChoice;
import org.colcon.burocrat.api.model.IFieldType;
import org.colcon.burocrat.api.model.IResponse;
import org.colcon.burocrat.api.utils.ArrayUtil;
import org.colcon.burocrat.model.FieldType;
import org.colcon.burocrat.model.Model;
import org.colcon.burocrat.model.Response;

public enum AppArg implements IArg
{
	;
	
	private IArg parentArg;
	private IFieldType fieldType;
	private String description;
	
	private Set<IFlag> flags;
	
	private AppArg(String description)
	{
		this(null, FieldType.TEXT_BOX, description);
	}
	
	private AppArg(IArg parentArg, String description)
	{
		this(parentArg, FieldType.TEXT_BOX, description);
	}
	
	private AppArg(IFieldType inputType, String description)
	{
		this(null, inputType, description);
	}
	
	private AppArg(IArg parentArg, IFieldType fieldType, String description)
	{
		this(parentArg, fieldType, description, new IFlag[0]);
	}
	
	private AppArg(IFieldType inputType, String description, IFlag[] flags)
	{
		this(null, inputType, description, flags);
	}
	
	private AppArg(IArg parentArg, IFieldType fieldType, String description, IFlag[] flags)
	{
		this.parentArg = parentArg;
		this.fieldType = fieldType;
		this.description = description;
		
		this.flags = new HashSet<IFlag>();
		this.flags.addAll(Arrays.asList(flags));
	}
	
	public IArg parent()
	{
		return parentArg;
	}
	
	public IFieldType fieldType()
	{
		if(parentArg != null)
			return parentArg.fieldType();
		
		return fieldType;
	}
	
	public String description()
	{
		return description;
	}
	
	public IFlag[] flags()
	{
		if(parentArg != null)
			return ArrayUtil.append(parentArg.flags(), flags.toArray(new IFlag[0]));
		
		return flags.toArray(new IFlag[0]);
	}
	
	public boolean hasFlag(IFlag flag)
	{
		return flags.contains(flag);
	}
	
	public IChoice[] choices()
	{
		if(parentArg != null)
			return parentArg.choices();
		
		return null;
	}
	
	public IArgList subArgs()
	{
		if(parentArg != null)
			return parentArg.subArgs();
		
		return null;
	}
	
	public IArgList	subArgs(Map<IArg, String> input)
	{
		return null;
	}
	
	public IResponse validate(String value)
	{
		if(parentArg != null && !parentArg.validate(value).isSuccess())
			return new Response(false);
		
		if(value == null)
			return new Response(false);
		
		IChoice[] choiceArray = choices();
		
		if(choiceArray != null)
		{
			for(IChoice choice : choiceArray)
				if(choice.name().equalsIgnoreCase(value))
					return new Response(true);
		
			return new Response(false);
		}
		
		IArgList subArgs = subArgs();
		
		if(subArgs != null)
		{
			Map<IArg, String> valueMap = Model.instance.parser().parseArgs(value);
			
			return subArgs.validate(valueMap);
		}
		
		return new Response(true);
	}

	public Object parse(String value) 
	{
		if(parentArg != null)
			return parentArg.parse(value);
		
		return value;
	}
	
	public String getHelpString()
	{
		String str = name().toLowerCase();
		
		while(str.length() < 15)
			str += ' ';
		
		str += description();
		
		return str;
	}
	
	public static IArg[] getValues()
	{
		return (IArg[])values();
	}
}
