package org.colcon.burocrat.app.model;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IArgList;
import org.colcon.burocrat.api.model.IFlag;
import org.colcon.burocrat.api.model.IItem;
import org.colcon.burocrat.api.model.IItemType;
import org.colcon.burocrat.api.utils.ArrayUtil;
import org.colcon.burocrat.model.Arg;
import org.colcon.burocrat.model.Constants;

public enum AppItemType implements IItemType
{
	;
	
	private String plural, description;
	private IItemType parent;
	
	private IArgList args;
	private IItemType[] containerTypes;
	private Set<IFlag> flags;
	
	private AppItemType(String plural, String description, IItemType parent, IArgList args, IItemType[] containers, IFlag[] flags)
	{
		this.plural = plural;
		this.description = description;
		this.parent = parent;
		this.args = args;
		this.containerTypes = containers;
		
		this.flags = new HashSet<IFlag>();
		this.flags.addAll(Arrays.asList(flags));
	}
	
	public String plural()
	{
		return plural;
	}
	
	public String description()
	{
		return description;
	}
	
	public IItemType parent()
	{
		return parent;
	}
	
	public IArgList args()
	{
		return args;
	}
	
	public IArgList getAllArgs()
	{
		if(parent != null)
			return parent.getAllArgs().append(args());
		
		return args();
	}
	
	public IItemType[] containerTypes()
	{
		return containerTypes;
	}
	
	public IItemType[] getAllContainerTypes()
	{
		if(parent != null)
			ArrayUtil.append(parent.getAllContainerTypes(), containerTypes());
		
		return containerTypes();
	}
	
	public IItem[] getInitItems(IItem instance) 
	{
		return new IItem[0];
	}
	
	public IItem[] getAllInitItems(IItem instance) 
	{
		if(parent != null)
			ArrayUtil.append(parent.getAllInitItems(instance), getInitItems(instance));
		
		return new IItem[0];
	}
	
	public IFlag[] flags()
	{
		if(parent() != null)
			return ArrayUtil.append(parent().flags(), flags.toArray(new IFlag[0]));
		
		return flags.toArray(new IFlag[0]);
	}
	
	public boolean hasFlag(IFlag flag)
	{
		return flags.contains(flag);
	}

	public void init(IItem instance) 
	{
		if(parent != null)
			parent.init(instance);
		
		if(instance.getId() == null)	
			instance.setArg(Arg.ID, generateId(instance));
		
		if(instance.getParent() == null)
			instance.setArg(Arg.PARENT, Constants.ROOT);
	}
	
	public String generateId(IItem instance)
	{
		if(parent != null)
			return parent.generateId(instance);
		
		return null;
	}
	
	public Map<IArg, String> withArgs(Object... values)
	{
		Map<IArg, String> map = getAllArgs().withArgs(values);
		
		map.put(Arg.TYPE, name());
		
		return map;
	}
	
	public static IItemType[] getValues()
	{
		return (IItemType[])values();
	}
}


