package org.colcon.burocrat.app.server;

import org.colcon.burocrat.api.model.IArg;
import org.colcon.burocrat.api.model.IItem;
import org.colcon.burocrat.api.model.IItemType;
import org.colcon.burocrat.api.server.IItemTypeImpl;
import org.colcon.burocrat.model.Arg;
import org.colcon.burocrat.model.ArgFlag;
import org.colcon.burocrat.model.Model;
import org.colcon.burocrat.server.Server;

public enum AppItemTypeImpl implements IItemTypeImpl
{
	;
	
	public IItemTypeImpl getParent()
	{
		IItemType type = Model.instance.getItemType(name());
		
		if(type == null)
			return null;
		
		if(type.parent() == null)
			return null;
		
		return Server.instance.getItemTypeImpl(type.parent().name());
	}
	
	public boolean validate(IItem instance)
	{
		if(getParent() != null)
			return getParent().validate(instance);
		
		return instance != null && instance.getType() != null && instance.getPath() != null;
	}
	
	public void init(IItem instance)
	{
		if(getParent() != null)
			getParent().init(instance);
		
		if(instance.getId() == null)
			instance.setArg(Arg.ID, generateId(instance));
		
		// encrypt args with ENCRYPTED flag
		for(IArg arg : instance.getArgKeys())
		{
			if(arg.hasFlag(ArgFlag.ENCRYPTED))
			{
				String plain = instance.getArg(arg);
				String encrypted = Server.instance.encryptor().encrypt(plain);
				instance.setArg(arg, encrypted);
			}
		}
	}
	
	public String generateId(IItem instance)
	{
		if(getParent() != null)
			return getParent().generateId(instance);
		
		String id = (int)(Math.random() * 10) + "";
		instance.setArg(Arg.ID, id);
		
		while(Server.instance.get(instance.getPath()) != null)
		{
			id += (int)(Math.random() * 10) + "";
			instance.setArg(Arg.ID, id);
		}
		
		return id;
	}
	
	public static IItemTypeImpl[] getValues()
	{
		return (IItemTypeImpl[])values();
	}
}
